﻿using DbContextLib;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Model.Business;
using Model.DTO;
using Services;

namespace API.Controllers
{
    [Route("api/[controller]")]
    [Authorize]
    [ApiController]
    public class SuccessesController : Controller
    {
        private IDataService _successDataService;

        private readonly ILogger<SuccessesController> _logger;

        public SuccessesController(IDataService successDataService, ILogger<SuccessesController> logger)
        {
            _successDataService = successDataService;
            _logger = logger;
        }

        [HttpGet("successes/{page}/{number}")]
        public async Task<IActionResult> GetSuccesses(int page, int number) 
        {
            var nbUser = (await _successDataService.GetUsers(page, number)).ToList().Count();
            if(nbUser == 0) 
            {
                _logger.LogError("[ERREUR] Aucun utilisateur trouvé.");
                return StatusCode(204);
            }
            _logger.LogInformation("[INFORMATION] {nb} Utilisateur(s) trouvé(s)", nbUser);
            return Ok(_successDataService.GetUsers(page, number));
        }

        [HttpGet("success/user/{id}")]
        public async Task<IActionResult> GetSuccessByUserId(int userId)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] Le succès avec l'id de l'utilisateur {id} a été trouvé.", userId);
                return Ok(_successDataService.GetSuccessByUserId(userId));
            } catch (ArgumentException) 
            {
                _logger.LogError("[ERREUR] Aucun utilisateur trouvé avec l'id de l'utilisateur {id}.", userId);
                return NotFound();
            }
        }

        [HttpGet("success/inquiry/{inquiryId}")]
        public async Task<IActionResult> GetSuccessByInquiryId(int inquiryId)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] Utilisateur avec l'id de l'enquête {inquiryId} a été trouvé.", inquiryId);
                return Ok(_successDataService.GetSuccessByInquiryId(inquiryId));
            }catch (ArgumentException) 
            {
                _logger.LogError("[ERREUR] Aucun utilisateur trouvé avec l'id de l'enquête {inquiryId}.", inquiryId);
                return NotFound();
            }
            
        }

        [HttpDelete]
        public async Task<IActionResult> DeleteSuccess(int id)
        {
            var success = await _successDataService.DeleteSuccess(id);
            if(success)
            {
                _logger.LogInformation("[INFORMATION] Le succès avec l'id {id} a été supprimé.", id);
                return Ok(_successDataService.DeleteSuccess(id));
            } else
            {
                _logger.LogError("[ERREUR] Aucun succès trouvé avec l'id {id}.", id);
                return NotFound();    
            }
            
        }

        [HttpPost]
        public async Task<IActionResult> CreateSuccess([FromBody] SuccessDTO dto)
        {
            /*if (dto.UserId == null || dto.InquiryId == null)
            {
                return BadRequest();
            }*/
            _logger.LogInformation("[INFORMATION] Un succès a été créé : userId - {userId}, inquiryId - {inquiryId}, isFinished - {isFinished}", dto.UserId, dto.InquiryId, dto.IsFinished);
            return Created(nameof(GetSuccesses), _successDataService.CreateSuccess(dto.UserId, dto.InquiryId, dto.IsFinished));
        }

        [HttpPut]
        public async Task<IActionResult> UpdateSuccess(int id, [FromBody] SuccessDTO successDTO)
        {
            if(id !=  successDTO.UserId)
            {
                _logger.LogError("[ERREUR] Problème ID - La mise à jour du succès avec l'id de l'utilisateur {id} a échouée.", id);
                return BadRequest();
            }
            if(!ModelState.IsValid)
            {
                _logger.LogError("[ERREUR] Problème controlleur - La mise à jour du succès avec l'id de l'utilisateur {id} a échouée.", id);
                return BadRequest();
            }
            if(successDTO != null) 
            {
                _logger.LogInformation("[INFORMATION] La mise à jour du succès avec l'id de l'utilisateur {id} a été effectuée", id);
                return Ok(_successDataService.UpdateSuccess(id, successDTO));
            }
            _logger.LogError("[ERREUR] Aucun succès trouvé avec l'id de l'utilisateur {id}.", id);
            return NotFound(); 
        }
    }
}
